@@ -24,7 +24,6 @@
 #include "mul.h"
 #include "archive.h"
 #include "operators.h"
-#include "matrix.h"
 #include "utils.h"
 #include "clifford.h"
 #include "ncmul.h"
@@ -207,8 +206,8 @@ void add::do_print_csrc(const print_csrc & c, unsigned level) const
 		c.s << separator;
 		if (elem.coeff.is_equal(_ex1) || elem.coeff.is_equal(_ex_1)) {
 			elem.rest.print(c, precedence());
-		} else if (ex_to<numeric>(elem.coeff).numer().is_equal(*_num1_p) ||
-				 ex_to<numeric>(elem.coeff).numer().is_equal(*_num_1_p))
+		} else if (ex_to<numeric>(elem.coeff).numer().is_one()
+                        or ex_to<numeric>(elem.coeff).numer().is_minus_one())
 		{
 			elem.rest.print(c, precedence());
 			c.s << '/';
@@ -233,7 +232,7 @@ void add::do_print_csrc(const print_csrc & c, unsigned level) const
 		c.s << ")";
 }
 
-void add::do_print_python_repr(const print_python_repr & c, unsigned) const
+void add::do_print_python_repr(const print_python_repr & c, unsigned /*level*/) const
 {
 	c.s << class_name() << '(';
 	op(0).print(c);
@@ -256,7 +255,6 @@ bool add::info(unsigned inf) const
 		case info_flags::integer:
 		case info_flags::crational:
 		case info_flags::cinteger:
-		case info_flags::positive:
 		case info_flags::nonnegative:
 		case info_flags::posint:
 		case info_flags::nonnegint:
@@ -281,6 +279,21 @@ bool add::info(unsigned inf) const
 			}
 			return false;
 		}
+		case info_flags::positive: {
+                        if (not overall_coeff.info(info_flags::nonnegative))
+                                return false;
+                        bool positive_seen = overall_coeff.info(info_flags::positive);
+			for (const auto & elem : seq) {
+				ex t = recombine_pair_to_ex(elem);
+                                bool is_pos = t.info(info_flags::positive);
+                                if (not is_pos
+                                    and not t.info(info_flags::nonnegative))
+					return false;
+                                else if (not positive_seen and is_pos)
+                                        positive_seen = true;
+			}
+                        return positive_seen;
+                }
 	}
 	return inherited::info(inf);
 }
@@ -339,13 +352,13 @@ ex add::coeff(const ex & s, int n) const
  		if (!restcoeff.is_zero()) {
  			if (do_clifford) {
  				if (clifford_max_label(restcoeff) == -1) {
- 					coeffseq_cliff.push_back(combine_ex_with_coeff_to_pair(ncmul(restcoeff, dirac_ONE(rl)), elem.coeff));
+                                        coeffseq_cliff.push_back(expair(ncmul(restcoeff, dirac_ONE(rl)), elem.coeff));
 				} else {
- 					coeffseq_cliff.push_back(combine_ex_with_coeff_to_pair(restcoeff, elem.coeff));
+                                        coeffseq_cliff.push_back(expair(restcoeff, elem.coeff));
 					nonscalar = true;
  				}
 			}
-			coeffseq.push_back(combine_ex_with_coeff_to_pair(restcoeff, elem.coeff));
+			coeffseq.push_back(expair(restcoeff, elem.coeff));
 		}
 	}
 
@@ -362,9 +375,15 @@ ex add::coeff(const ex & s, int n) const
  *  @param level cut-off in recursive evaluation */
 ex add::eval(int level) const
 {
+        if ((level == 1) && (flags & status_flags::evaluated)) {
+                GINAC_ASSERT(seq.size()>0);
+                GINAC_ASSERT(seq.size()>1 || !overall_coeff.is_zero());
+                return *this;
+        }
+
 	std::unique_ptr<epvector> evaled_seqp = evalchildren(level);
-	if (evaled_seqp.get()) {
-		// do more evaluation later
+	if (unlikely(evaled_seqp != nullptr)) {
+		// start over evaluating a new object
 		return (new add(*evaled_seqp, overall_coeff))->
 		       setflag(status_flags::dynallocated);
 	}
@@ -378,12 +397,6 @@ ex add::eval(int level) const
 	}
 #endif // def DO_GINAC_ASSERT
 	
-	if (flags & status_flags::evaluated) {
-		GINAC_ASSERT(seq.size()>0);
-		GINAC_ASSERT(seq.size()>1 || !overall_coeff.is_zero());
-		return *this;
-	}
-		
 	// handle infinity
         for (auto i = seq.begin(); i != seq.end(); i++)
                 if (unlikely(is_exactly_a<infinity>(i->rest)))
@@ -407,7 +420,7 @@ ex add::eval(int level) const
 	for (const auto & elem : seq)
 		if (unlikely(is_exactly_a<numeric>(elem.rest)))
 			++terms_to_collect;
-	if (terms_to_collect) {
+	if (terms_to_collect != 0) {
                 epvector s;
 		s.reserve(seq_size - terms_to_collect);
 		numeric oc = *_num0_p;
@@ -452,59 +465,27 @@ ex add::eval_infinity(epvector::const_iterator infinity_iter) const
 }
 
 
-ex add::evalm() const
-{
-	// Evaluate children first and add up all matrices. Stop if there's one
-	// term that is not a matrix.
-	epvector s;
-	s.reserve(seq.size());
-
-	bool all_matrices = true;
-	bool first_term = true;
-	matrix sum;
-
-	for (const auto & elem : seq) {
-		const ex &m = recombine_pair_to_ex(elem).evalm();
-		s.push_back(split_ex_to_pair(m));
-		if (is_exactly_a<matrix>(m)) {
-			if (first_term) {
-				sum = ex_to<matrix>(m);
-				first_term = false;
-			} else
-				sum = sum.add(ex_to<matrix>(m));
-		} else
-			all_matrices = false;
-	}
-
-	if (all_matrices)
-		return sum + overall_coeff;
-	else
-		return (new add(s, overall_coeff))->setflag(status_flags::dynallocated);
-}
-
 ex add::conjugate() const
 {
-	std::unique_ptr<exvector> v(nullptr);
-	for (size_t i=0; i<nops(); ++i) {
-		if (v) {
-			v->push_back(op(i).conjugate());
-			continue;
-		}
-		ex term = op(i);
-		ex ccterm = term.conjugate();
-		if (are_ex_trivially_equal(term, ccterm))
-			continue;
-		v.reset(new exvector);
-		v->reserve(nops());
-		for (size_t j=0; j<i; ++j)
-			v->push_back(op(j));
-		v->push_back(ccterm);
-	}
-	if (v) {
-		ex result = add(*v);
-		return result;
-	}
-	return *this;
+        for (const auto& elem : seq) {
+                ex term = recombine_pair_to_ex(elem);
+                if (not are_ex_trivially_equal(term, term.conjugate())) {
+                        epvector v;
+                        v.reserve(seq.size());
+                        bool first_ccterm_seen = false;
+                        for (const auto& elem1 : seq) {
+                                if (first_ccterm_seen or &elem1 == &elem) {
+                                        v.push_back(split_ex_to_pair(recombine_pair_to_ex(elem1).conjugate()));
+                                        first_ccterm_seen = true;
+                                }
+                                else
+                                        v.push_back(elem1);
+                        }
+                        return (new add(v, overall_coeff.conjugate()))
+                                -> setflag(status_flags::dynallocated);
+                }
+        }
+        return *this;
 }
 
 ex add::real_part() const
@@ -564,7 +545,7 @@ ex add::derivative(const symbol & y) const
 	// than the default implementation in basic::derivative() although
 	// if performs the same function (differentiate each term).
 	for (const auto & elem : seq)
-		s.push_back(combine_ex_with_coeff_to_pair(elem.rest.diff(y), elem.coeff));
+		s.push_back(expair(elem.rest.diff(y), elem.coeff));
 	return (new add(s, _ex0))->setflag(status_flags::dynallocated);
 }
 
@@ -590,13 +571,13 @@ tinfo_t add::return_type_tinfo() const
 }
 
 // Note: do_index_renaming is ignored because it makes no sense for an add.
-ex add::thisexpairseq(const epvector & v, const ex & oc, bool) const
+ex add::thisexpairseq(const epvector & v, const ex & oc, bool /*do_index_renaming*/) const
 {
 	return (new add(v,oc))->setflag(status_flags::dynallocated);
 }
 
 //// Note: do_index_renaming is ignored because it makes no sense for an add.
-ex add::thisexpairseq(std::unique_ptr<epvector> vp, const ex & oc, bool) const
+ex add::thisexpairseq(std::unique_ptr<epvector> vp, const ex & oc, bool /*do_index_renaming*/) const
 {
 	return (new add(*vp,oc))->setflag(status_flags::dynallocated);
 }
@@ -606,7 +587,9 @@ expair add::split_ex_to_pair(const ex & e) const
 	if (is_exactly_a<mul>(e)) {
 		const mul &mulref(ex_to<mul>(e));
 		const ex &numfactor = mulref.overall_coeff;
-		auto mulcopyp = new mul(mulref);
+		if (numfactor.is_integer_one())
+                        return expair(e, _ex1);
+                auto mulcopyp = new mul(mulref);
 		mulcopyp->overall_coeff = _ex1;
 		mulcopyp->clearflag(status_flags::evaluated);
 		mulcopyp->clearflag(status_flags::hash_calculated);
@@ -623,20 +606,22 @@ expair add::combine_ex_with_coeff_to_pair(const ex & e,
 	if (is_exactly_a<mul>(e)) {
 		const mul &mulref(ex_to<mul>(e));
 		const ex &numfactor = mulref.overall_coeff;
+		if (likely(numfactor.is_integer_one()))
+                        return expair(e, c);
 		auto mulcopyp = new mul(mulref);
 		mulcopyp->overall_coeff = _ex1;
 		mulcopyp->clearflag(status_flags::evaluated);
 		mulcopyp->clearflag(status_flags::hash_calculated);
 		mulcopyp->setflag(status_flags::dynallocated);
-		if (c.is_equal(_ex1))
+		if (c.is_integer_one())
 			return expair(*mulcopyp, numfactor);
-		else if (numfactor.is_equal(_ex1))
-			return expair(*mulcopyp, c);
 		else
 			return expair(*mulcopyp, ex_to<numeric>(numfactor).mul_dyn(ex_to<numeric>(c)));
 	} else if (is_exactly_a<numeric>(e)) {
-		if (c.is_equal(_ex1))
+		if (c.is_integer_one())
 			return expair(e, _ex1);
+		if (e.is_integer_one())
+			return expair(c, _ex1);
 		return expair(ex_to<numeric>(e).mul_dyn(ex_to<numeric>(c)), _ex1);
 	}
 	return expair(e, c);
@@ -649,7 +634,7 @@ expair add::combine_pair_with_coeff_to_pair(const expair & p,
 	GINAC_ASSERT(is_exactly_a<numeric>(c));
 
 	if (is_exactly_a<numeric>(p.rest)) {
-		GINAC_ASSERT(ex_to<numeric>(p.coeff).is_equal(*_num1_p)); // should be normalized
+		GINAC_ASSERT(ex_to<numeric>(p.coeff).is_one()); // should be normalized
 		return expair(ex_to<numeric>(p.rest).mul_dyn(ex_to<numeric>(c)),_ex1);
 	}
 
@@ -658,7 +643,7 @@ expair add::combine_pair_with_coeff_to_pair(const expair & p,
 
 ex add::recombine_pair_to_ex(const expair & p) const
 {
-	if (ex_to<numeric>(p.coeff).is_equal(*_num1_p))
+	if (ex_to<numeric>(p.coeff).is_one())
 		return p.rest;
 	else
 		return (new mul(p.rest,p.coeff))->setflag(status_flags::dynallocated);
@@ -667,7 +652,7 @@ ex add::recombine_pair_to_ex(const expair & p) const
 ex add::expand(unsigned options) const
 {
 	std::unique_ptr<epvector> vp = expandchildren(options);
-	if (vp.get() == nullptr) {
+	if (vp == nullptr) {
 		// the terms have not changed, so it is safe to declare this expanded
 		return (options == 0) ? setflag(status_flags::expanded) : *this;
 	}
@@ -696,4 +681,55 @@ ex add::lead_coeff() const {
 	}
 }
 
+ex add::combine_fractions() const
+{
+        epvector rseq;
+        exmap map;
+        ex oc = overall_coeff;
+        for (const auto& pair : seq) {
+                const ex &e = recombine_pair_to_ex(pair);
+                if (is_exactly_a<power>(e)) {
+                        const power& p = ex_to<power>(e);
+                        const ex& eexp = p.op(1);
+                        if (eexp.info(info_flags::negative)) {
+                                auto it = map.find(p);
+                                if (it != map.end())
+                                        it->second += _ex1;
+                                else
+                                        map[p] = _ex1;
+                        }
+                        else
+                                rseq.push_back(pair);
+                }
+                else if (is_exactly_a<mul>(e)) {
+                        const mul& m = ex_to<mul>(e);
+                        epvector denseq, coseq;
+                        for (const auto& mpair : m.seq) {
+                                if (mpair.coeff.info(info_flags::negative))
+                                        denseq.push_back(mpair);
+                                else
+                                        coseq.push_back(mpair);
+                        }
+                        if (not denseq.empty()) {
+                                mul den(denseq);
+                                auto it = map.find(den);
+                                mul coeff(coseq, pair.coeff);
+                                if (it != map.end()) {
+                                        it->second += coeff;
+                                }
+                                else
+                                        map[den] = coeff;
+                        }
+                        else
+                                rseq.push_back(pair);
+                }
+                else
+                        rseq.push_back(pair);
+        }
+        for (const auto& t : map)
+                rseq.push_back(split_ex_to_pair(mul(t.first, t.second)));
+        add rex(rseq);
+        return add(rex, oc);
+}
+
 } // namespace GiNaC
