@@ -459,20 +459,6 @@ ex basic::evalm() const
 	return *this;
 }
 
-/** Function object to be applied by basic::eval_integ(). */
-struct eval_integ_map_function : public map_function {
-	ex operator()(const ex & e) override { return eval_integ(e); }
-} map_eval_integ;
-
-/** Evaluate integrals, if result is known. */
-ex basic::eval_integ() const
-{
-	if (nops() == 0)
-		return *this;
-	else
-		return map(map_eval_integ);
-}
-
 /** Perform automatic symbolic evaluations on indexed expression that
  *  contains this object as the base expression. */
 ex basic::eval_indexed(const basic & i) const
@@ -775,7 +761,7 @@ tinfo_t basic::return_type_tinfo() const
  *  would all end up with the same hashvalue. */
 long basic::calchash() const
 {
-	long v = golden_ratio_hash((p_int)tinfo());
+	long v = golden_ratio_hash((intptr_t)tinfo());
 	for (size_t i=0; i<nops(); i++) {
 		v = rotate_left(v);
 		v ^= this->op(i).gethash();
