@@ -196,7 +196,7 @@ bool clifford::same_metric(const ex & other) const
 // functions overriding virtual functions from base classes
 //////////
 
-ex clifford::op(size_t i) const
+const ex clifford::op(size_t i) const
 {
 	GINAC_ASSERT(i<nops());
 	if (nops()-i == 1)
@@ -338,7 +338,7 @@ struct is_not_a_clifford : public std::unary_function<ex, bool> {
 };
 
 /** Contraction of a gamma matrix with something else. */
-bool diracgamma::contract_with(exvector::iterator self, exvector::iterator other, exvector &) const
+bool diracgamma::contract_with(exvector::iterator self, exvector::iterator other, exvector & /*v*/) const
 {
 	GINAC_ASSERT(is_a<clifford>(*self));
 	GINAC_ASSERT(is_a<indexed>(*other));
@@ -397,7 +397,7 @@ bool diracgamma::contract_with(exvector::iterator self, exvector::iterator other
 
 		// gamma~mu Sodd gamma.mu = -2 Sodd_R
 		// (Chisholm identity in 4 dimensions)
-		} else if (!((other - self) & 1) && dim.is_equal(4)) {
+		} else if ((((other - self) & 1) == 0) && dim.is_equal(4)) {
 			if (std::find_if(self + 1, other, is_not_a_clifford()) != other)
 				return false;
 
@@ -409,7 +409,7 @@ bool diracgamma::contract_with(exvector::iterator self, exvector::iterator other
 		// gamma~mu Sodd gamma~alpha gamma.mu = 2 gamma~alpha Sodd + 2 Sodd_R gamma~alpha
 		// (commutate contracted indices towards each other, then use
 		// Chisholm identity in 4 dimensions)
-		} else if (((other - self) & 1) && dim.is_equal(4)) {
+		} else if ((((other - self) & 1) != 0) && dim.is_equal(4)) {
 			if (std::find_if(self + 1, other, is_not_a_clifford()) != other)
 				return false;
 
@@ -449,7 +449,7 @@ bool diracgamma::contract_with(exvector::iterator self, exvector::iterator other
 }
 
 /** Contraction of a Clifford unit with something else. */
-bool cliffordunit::contract_with(exvector::iterator self, exvector::iterator other, exvector &) const
+bool cliffordunit::contract_with(exvector::iterator self, exvector::iterator other, exvector & /*v*/) const
 {
 	GINAC_ASSERT(is_a<clifford>(*self));
 	GINAC_ASSERT(is_a<indexed>(*other));
@@ -800,15 +800,15 @@ ex dirac_slash(const ex & e, const ex & dim, unsigned char rl)
  *  is that of a clifford object (with an arbitrary representation label). */
 bool is_clifford_tinfo(tinfo_t ti)
 {
-	p_int start_loc=(p_int)&clifford::return_type_tinfo_static;
-	return (p_int)ti>=start_loc && (p_int)ti<start_loc+256;
+	intptr_t start_loc=(intptr_t)&clifford::return_type_tinfo_static;
+	return (intptr_t)ti>=start_loc && (intptr_t)ti<start_loc+256;
 }
 
 /** Extract representation label from tinfo key (as returned by
  *  return_type_tinfo()). */
 static unsigned char get_representation_label(tinfo_t ti)
 {
-	return (unsigned char)((p_int)ti-(p_int)&clifford::return_type_tinfo_static);
+	return (unsigned char)((intptr_t)ti-(intptr_t)&clifford::return_type_tinfo_static);
 }
 
 /** Take trace of a string of an even number of Dirac gammas given a vector
@@ -1055,7 +1055,7 @@ ex canonicalize_clifford(const ex & e_)
 						base_and_index(it[1], b2, i2);
 						// for Clifford algebras (commutator_sign == -1) metric should be symmetrised
 						it[0] = (ex_to<clifford>(save0).get_metric(i1, i2, ex_to<clifford>(save0).get_commutator_sign() == -1) * b1 * b2).simplify_indexed();
-						it[1] = v.size() ? _ex2 * dirac_ONE(ex_to<clifford>(save0).get_representation_label()) : _ex2;
+						it[1] = v.size() != 0u ? _ex2 * dirac_ONE(ex_to<clifford>(save0).get_representation_label()) : _ex2;
 						ex sum = ncmul(v);
 						it[0] = save1;
 						it[1] = save0;
@@ -1091,8 +1091,8 @@ ex remove_dirac_ONE(const ex & e, unsigned char rl, unsigned options)
 	pointer_to_map_function_2args<unsigned char, unsigned> fcn(remove_dirac_ONE, rl, options | 1);
 	bool need_reevaluation = false;
 	ex e1 = e;
-	if (! (options & 1) )  { // is not a child
-		if (options & 2)
+	if ((options & 1) == 0u )  { // is not a child
+		if ((options & 2) != 0u)
 			e1 = expand_dummy_sum(e, true);
 		e1 = canonicalize_clifford(e1);
 	}
@@ -1104,7 +1104,7 @@ ex remove_dirac_ONE(const ex & e, unsigned char rl, unsigned options)
 			throw(std::invalid_argument("remove_dirac_ONE(): expression is a non-scalar Clifford number!"));
 	} else if (is_a<add>(e1) || is_a<ncmul>(e1) || is_a<mul>(e1)  
 			   || is_a<matrix>(e1) || e1.info(info_flags::list)) {
-		if (options & 3) // is a child or was already expanded
+		if ((options & 3) != 0u) // is a child or was already expanded
 			return e1.map(fcn);
 		else
 			try {
@@ -1113,7 +1113,7 @@ ex remove_dirac_ONE(const ex & e, unsigned char rl, unsigned options)
 				need_reevaluation = true;
 			}
 	} else if (is_a<power>(e1)) {
-		if (options & 3) // is a child or was already expanded
+		if ((options & 3) != 0u) // is a child or was already expanded
 			return pow(remove_dirac_ONE(e1.op(0), rl, options | 1), e1.op(1));
 		else
 			try {
