@@ -20,6 +20,7 @@
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
+#include <Python.h>
 #include "ex.h"
 #include "symbol.h"
 #include "add.h"
@@ -29,6 +30,7 @@
 #include "power.h"
 #include "lst.h"
 #include "function.h"
+#include "fderivative.h"
 #include "symbol.h"
 #include "relational.h"
 #include "utils.h"
@@ -36,6 +38,7 @@
 
 #include <iostream>
 #include <stdexcept>
+#include <unordered_map>
 
 namespace GiNaC {
 
@@ -319,7 +322,7 @@ bool ex::is_integer_one() const
     if (!is_exactly_a<numeric>(*this))
         return false;
     return ex_to<numeric>(*this).is_integer()
-        and ex_to<numeric>(*this).is_equal(*_num1_p);
+        and ex_to<numeric>(*this).is_one();
 }
   
 bool ex::is_integer_pmone() const
@@ -328,8 +331,8 @@ bool ex::is_integer_pmone() const
         return false;
     numeric num = ex_to<numeric>(*this);
     return ((ex_to<numeric>(*this).is_integer())
-        and (ex_to<numeric>(*this).is_equal(*_num1_p)
-            or ex_to<numeric>(*this).is_equal(*_num_1_p)));
+        and (ex_to<numeric>(*this).is_one()
+            or ex_to<numeric>(*this).is_minus_one()));
 }
 
 void ex::set_domain(unsigned d)
@@ -345,6 +348,7 @@ void ex::set_domain(unsigned d)
 }
 
 size_t ex::nsymbols() const
+// DON'T USE if you want the number of symbols, instead use symbols().size()
 {
 	int res = 0;
 	if (is_exactly_a<symbol>(*this)) {
@@ -395,6 +399,54 @@ symbolset ex::symbols() const
 	return the_set;
 }
 
+static void collect_bound_symbols(const ex& e, symbolset& syms)
+{
+        static unsigned int sum_serial = function::find_function("sum", 4);
+        static unsigned int integral_serial = function::find_function("integrate", 4);
+        static unsigned int limit_serial = function::find_function("limit", 0);
+	if (is_exactly_a<function>(e)) {
+                const function& f = ex_to<function>(e);
+                if (f.get_serial() == sum_serial
+                    and is_exactly_a<symbol>(f.op(1))) {
+                        syms.insert(ex_to<symbol>(f.op(1)));
+                        return collect_bound_symbols(f.op(0), syms);
+                }
+                if (f.get_serial() == integral_serial
+                    and is_exactly_a<symbol>(f.op(1))) {
+                        syms.insert(ex_to<symbol>(f.op(1)));
+                        return collect_bound_symbols(f.op(0), syms);
+                }
+                if (f.get_serial() == limit_serial
+                    and is_exactly_a<symbol>(f.op(1))) {
+                        syms.insert(ex_to<symbol>(f.op(1)));
+                        return collect_bound_symbols(f.op(0), syms);
+                }
+        }
+        else if (is_exactly_a<fderivative>(e)) {
+                const fderivative& d = ex_to<fderivative>(e);
+                for (size_t i=0; i<d.nops(); i++)
+                        if (is_exactly_a<symbol>(d.op(i)))
+                                syms.insert(ex_to<symbol>(d.op(i)));
+                return;
+        }
+	else
+		for (size_t i=0; i < e.nops(); i++)
+                        collect_bound_symbols(e.op(i), syms);
+}
+
+symbolset ex::free_symbols() const
+{
+        symbolset the_set, bound_set;
+        collect_symbols(*this, the_set);
+        collect_bound_symbols(*this, bound_set);
+        for (auto it = the_set.begin(); it != the_set.end(); )
+                if (bound_set.find(*it) != bound_set.end())
+                        it = the_set.erase(it);
+                else
+                        ++it;
+	return the_set;
+}
+
 ex ex::sorted_op(size_t i) const
 {
 	if (is_a<expairseq>(*this))
