@@ -270,11 +270,11 @@ bool expairseq::info(unsigned inf) const
 {
 	switch(inf) {
 		case info_flags::expanded:
-			return (flags & status_flags::expanded);
+			return (flags & status_flags::expanded) != 0u;
 		case info_flags::has_indices: {
-			if (flags & status_flags::has_indices)
+			if ((flags & status_flags::has_indices) != 0u)
 				return true;
-			else if (flags & status_flags::has_no_indices)
+			else if ((flags & status_flags::has_no_indices) != 0u)
 				return false;
 			for (const auto & elem : seq) {
 				if (elem.rest.info(info_flags::has_indices)) {
@@ -299,7 +299,7 @@ size_t expairseq::nops() const
 		return seq.size()+1;
 }
 
-ex expairseq::op(size_t i) const
+const ex expairseq::op(size_t i) const
 {
 	if (i < seq.size())
 		return recombine_pair_to_ex(seq[i]);
@@ -340,11 +340,11 @@ ex expairseq::map(map_function &f) const
 /** Perform coefficient-wise automatic term rewriting rules in this class. */
 ex expairseq::eval(int level) const
 {
-	if ((level==1) && (flags &status_flags::evaluated))
+	if ((level==1) && ((flags &status_flags::evaluated) != 0u))
 		return *this;
 	
 	std::unique_ptr<epvector> vp = evalchildren(level);
-	if (vp.get() == nullptr)
+	if (vp == nullptr)
 		return this->hold();
 	
 	return (new expairseq(std::move(vp), overall_coeff))->setflag(status_flags::dynallocated | status_flags::evaluated);
@@ -354,7 +354,7 @@ epvector* conjugateepvector(const epvector& epv)
 {
 	epvector *newepv = nullptr;
 	for (const auto & elem : epv) {
-		if (newepv) {
+		if (newepv != nullptr) {
 			newepv->push_back(elem.conjugate());
 			continue;
 		}
@@ -452,11 +452,10 @@ found:		;
 			// Assign all the remaining terms to the global wildcard (unless
 			// it has already been matched before, in which case the matches
 			// must be equal)
-			size_t num = ops.size();
 			std::unique_ptr<epvector> vp(new epvector);
-			vp->reserve(num);
-			for (size_t i=0; i<num; i++)
-				vp->push_back(split_ex_to_pair(ops[i]));
+			vp->reserve(ops.size());
+			for (const auto& term : ops)
+				vp->push_back(split_ex_to_pair(term));
 
                         ex rest = thisexpairseq(std::move(vp), default_overall_coeff());
                         for (const auto & elem : tmp_repl) {
@@ -489,9 +488,9 @@ found:		;
 ex expairseq::subs(const exmap & m, unsigned options) const
 {
 	std::unique_ptr<epvector> vp = subschildren(m, options);
-	if (vp.get())
+	if (vp.get() != nullptr)
 		return ex_to<basic>(thisexpairseq(std::move(vp), overall_coeff, (options & subs_options::no_index_renaming) == 0));
-	else if ((options & subs_options::algebraic) && is_exactly_a<mul>(*this))
+	else if (((options & subs_options::algebraic) != 0u) && is_exactly_a<mul>(*this))
 		return static_cast<const mul *>(this)->algebraic_subs_mul(m, options);
 	else
 		return subs_one_level(m, options);
@@ -644,7 +643,7 @@ unsigned expairseq::return_type() const
 
 long expairseq::calchash() const
 {
-	long v = golden_ratio_hash((p_int)this->tinfo());
+	long v = golden_ratio_hash((intptr_t)tinfo());
         for (const auto & elem : seq) {
 		v ^= elem.rest.gethash();
 #if !EXPAIRSEQ_USE_HASHTAB
@@ -657,7 +656,7 @@ long expairseq::calchash() const
 	v ^= overall_coeff.gethash();
 
 	// store calculated hash value only if object is already evaluated
-	if (flags &status_flags::evaluated) {
+	if ((flags &status_flags::evaluated) != 0u) {
 		setflag(status_flags::hash_calculated);
 		hashvalue = v;
 	}
@@ -668,7 +667,7 @@ long expairseq::calchash() const
 ex expairseq::expand(unsigned options) const
 {
 	std::unique_ptr<epvector> vp = expandchildren(options);
-	if (vp.get())
+	if (vp.get() != nullptr)
 		return thisexpairseq(std::move(vp), overall_coeff);
 	else {
 		// The terms have not changed, so it is safe to declare this expanded
@@ -700,7 +699,7 @@ ex expairseq::thisexpairseq(std::unique_ptr<epvector> vp, const ex &oc, bool do_
 	return expairseq(std::move(vp), oc, do_index_renaming);
 }
 
-void expairseq::printpair(const print_context & c, const expair & p, unsigned) const
+void expairseq::printpair(const print_context & c, const expair & p, unsigned /*unused*/) const
 {
 	c.s << "[[";
 	p.rest.print(c, precedence());
@@ -766,7 +765,7 @@ ex expairseq::recombine_pair_to_ex(const expair &p) const
 	return lst(p.rest,p.coeff);
 }
 
-bool expairseq::expair_needs_further_processing(epp)
+bool expairseq::expair_needs_further_processing(epp /*unused*/)
 {
 #if EXPAIRSEQ_USE_HASHTAB
 	//#  error "FIXME: expair_needs_further_processing not yet implemented for hashtabs, sorry. A.F."
@@ -801,7 +800,7 @@ void expairseq::combine_overall_coeff(const ex &c1, const ex &c2)
 	                add_dyn(ex_to<numeric>(c1).mul(ex_to<numeric>(c2)));
 }
 
-bool expairseq::can_make_flat(const expair &) const
+bool expairseq::can_make_flat(const expair & /*unused*/) const
 {
 	return true;
 }
@@ -1557,7 +1556,7 @@ void expairseq::combine_same_terms()
 bool expairseq::is_canonical() const
 {
 	if (seq.size() <= 1)
-		return 1;
+		return true;
 	
 #if EXPAIRSEQ_USE_HASHTAB
 	if (hashtabsize > 0) return 1; // not canoncalized
@@ -1582,12 +1581,12 @@ bool expairseq::is_canonical() const
 					std::clog << "pair2:" << std::endl;
 					it->rest.print(print_tree(std::clog));
 					it->coeff.print(print_tree(std::clog));
-					return 0;
+					return false;
 				}
 			}
 		}
 	}
-	return 1;
+	return true;
 }
 
 
@@ -1601,33 +1600,29 @@ std::unique_ptr<epvector> expairseq::expandchildren(unsigned options) const
 	auto cit = seq.begin();
 	auto last = seq.end();
 	while (cit!=last) {
-		const ex &expanded_ex = cit->rest.expand(options);
+		const ex expanded_ex = cit->rest.expand(options);
 		if (!are_ex_trivially_equal(cit->rest,expanded_ex)) {
 			
 			// something changed, copy seq, eval and return it
-			std::unique_ptr<epvector> s(new epvector);
+                        std::unique_ptr<epvector> s(new epvector);
 			s->reserve(seq.size());
 			
 			// copy parts of seq which are known not to have changed
-			auto cit2 = seq.begin();
-			while (cit2!=cit) {
-				s->push_back(*cit2);
-				++cit2;
-			}
+                        s->insert(s->begin(), seq.begin(), cit);
 
 			// copy first changed element
-			s->push_back(combine_ex_with_coeff_to_pair(expanded_ex,
-			                                           cit2->coeff));
-			++cit2;
+			s->push_back(expair(expanded_ex, cit->coeff));
+			++cit;
 
 			// copy rest
-			while (cit2!=last) {
-				s->push_back(combine_ex_with_coeff_to_pair(cit2->rest.expand(options),
-				                                           cit2->coeff));
-				++cit2;
+			while (cit != last) {
+				s->push_back(expair(cit->rest.expand(options),
+				                         cit->coeff));
+				++cit;
 			}
-			return std::move(s);
+			return s;
 		}
+
 		++cit;
 	}
 	
@@ -1642,46 +1637,35 @@ std::unique_ptr<epvector> expairseq::expandchildren(unsigned options) const
  *  if no members were changed. */
 std::unique_ptr<epvector> expairseq::evalchildren(int level) const
 {
-	// returns a NULL pointer if nothing had to be evaluated
-	// returns a pointer to a newly created epvector otherwise
-	// (which has to be deleted somewhere else)
-
-	if (level==1)
-		return std::unique_ptr<epvector>(nullptr);
-	
 	if (level == -max_recursion_level)
 		throw(std::runtime_error("max recursion level reached"));
 	
-	--level;
 	auto last = seq.end();
 	auto cit = seq.begin();
 	while (cit!=last) {
-		const ex &evaled_ex = cit->rest.eval(level);
-		if (!are_ex_trivially_equal(cit->rest,evaled_ex)) {
-			
-			// something changed, copy seq, eval and return it
-			std::unique_ptr<epvector> s(new epvector);
+                const ex evaled_rest = level==1 ? cit->rest : cit->rest.eval(level-1);
+                const expair evaled_pair = combine_ex_with_coeff_to_pair(evaled_rest, cit->coeff);
+                if (unlikely(!evaled_pair.is_equal(*cit))) {
+
+			// something changed: copy seq, eval, and return it
+                        std::unique_ptr<epvector> s(new epvector);
 			s->reserve(seq.size());
-			
+
 			// copy parts of seq which are known not to have changed
-			auto cit2=seq.begin();
-			while (cit2!=cit) {
-				s->push_back(*cit2);
-				++cit2;
-			}
+                        s->insert(s->begin(), seq.begin(), cit);
 
 			// copy first changed element
-			s->push_back(combine_ex_with_coeff_to_pair(evaled_ex,
-			                                           cit2->coeff));
-			++cit2;
+			s->push_back(evaled_pair);
+			++cit;
 
 			// copy rest
-			while (cit2!=last) {
-				s->push_back(combine_ex_with_coeff_to_pair(cit2->rest.eval(level),
-				                                           cit2->coeff));
-				++cit2;
+			while (cit != last) {
+                                const ex evaled_rest1 = level==1 ? cit->rest : cit->rest.eval(level-1);
+				s->push_back(combine_ex_with_coeff_to_pair(evaled_rest1,
+				                                       cit->coeff));
+				++cit;
 			}
-			return std::move(s);
+			return s;
 		}
 		++cit;
 	}
@@ -1699,7 +1683,7 @@ std::unique_ptr<epvector> expairseq::subschildren(const exmap & m, unsigned opti
 	// When any of the objects to be substituted is a product or power
 	// we have to recombine the pairs because the numeric coefficients may
 	// be part of the search pattern.
-	if (!(options & (subs_options::pattern_is_product | subs_options::pattern_is_not_product))) {
+	if ((options & (subs_options::pattern_is_product | subs_options::pattern_is_not_product)) == 0u) {
 
 		// Search the list of substitutions and cache our findings
 		for (const auto & elem : m) {
@@ -1708,21 +1692,21 @@ std::unique_ptr<epvector> expairseq::subschildren(const exmap & m, unsigned opti
 				break;
 			}
 		}
-		if (!(options & subs_options::pattern_is_product))
+		if ((options & subs_options::pattern_is_product) == 0u)
 			options |= subs_options::pattern_is_not_product;
 	}
 
-	if (options & subs_options::pattern_is_product) {
+	if ((options & subs_options::pattern_is_product) != 0u) {
 
 		// Substitute in the recombined pairs
 		auto cit = seq.begin(), last = seq.end();
-		while (cit != last) {
+                while (cit != last) {
 
-			const ex &orig_ex = recombine_pair_to_ex(*cit);
-			const ex &subsed_ex = orig_ex.subs(m, options);
-			if (!are_ex_trivially_equal(orig_ex, subsed_ex)) {
+                        const ex &orig_ex = recombine_pair_to_ex(*cit);
+                        const ex &subsed_ex = orig_ex.subs(m, options);
+                        if (!are_ex_trivially_equal(orig_ex, subsed_ex)) {
 
-				// Something changed, copy seq, subs and return it
+				// Something changed: copy seq, subs, and return it
 				std::unique_ptr<epvector> s(new epvector);
 				s->reserve(seq.size());
 
@@ -1738,7 +1722,7 @@ std::unique_ptr<epvector> expairseq::subschildren(const exmap & m, unsigned opti
 					s->push_back(split_ex_to_pair(recombine_pair_to_ex(*cit).subs(m, options)));
 					++cit;
 				}
-				return std::move(s);
+				return s;
 			}
 
 			++cit;
@@ -1749,11 +1733,10 @@ std::unique_ptr<epvector> expairseq::subschildren(const exmap & m, unsigned opti
 		// Substitute only in the "rest" part of the pairs
 		auto cit = seq.begin(), last = seq.end();
 		while (cit != last) {
-
 			const ex &subsed_ex = cit->rest.subs(m, options);
 			if (!are_ex_trivially_equal(cit->rest, subsed_ex)) {
-			
-				// Something changed, copy seq, subs and return it
+
+                                // Something changed, copy seq, subs and return it
 				std::unique_ptr<epvector> s(new epvector);
 				s->reserve(seq.size());
 
@@ -1769,7 +1752,7 @@ std::unique_ptr<epvector> expairseq::subschildren(const exmap & m, unsigned opti
 					s->push_back(combine_ex_with_coeff_to_pair(cit->rest.subs(m, options), cit->coeff));
 					++cit;
 				}
-				return std::move(s);
+				return s;
 			}
 
 			++cit;
