@@ -20,6 +20,8 @@
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
+#include <Python.h>
+#include "py_funcs.h"
 #include "function.h"
 #include "operators.h"
 #include "fderivative.h"
@@ -35,7 +37,6 @@
 #include "utils.h"
 #include "remember.h"
 #include "symbol.h"
-#include "py_funcs.h"
 
 #include <iostream>
 #include <string>
@@ -82,7 +83,7 @@ void function_options::initialize()
 	set_name(s1, s2);
 	nparams = 0;
 	eval_f = real_part_f = imag_part_f = conjugate_f = derivative_f
-		= power_f = series_f = nullptr;
+		= expl_derivative_f = power_f = series_f = nullptr;
 	evalf_f = nullptr;
 	evalf_params_first = true;
 	apply_chain_rule = true;
@@ -93,6 +94,7 @@ void function_options::initialize()
 	real_part_use_exvector_args = false;
 	imag_part_use_exvector_args = false;
 	derivative_use_exvector_args = false;
+        expl_derivative_use_exvector_args = false;
 	power_use_exvector_args = false;
 	series_use_exvector_args = false;
 	print_use_exvector_args = false;
@@ -255,6 +257,25 @@ function_options & function_options::derivative_func(derivative_funcp_6 d)
 	return *this;
 }
 
+function_options & function_options::expl_derivative_func(expl_derivative_funcp_1 d)
+{
+	test_and_set_nparams(1);
+	expl_derivative_f = expl_derivative_funcp(d);
+	return *this;
+}
+function_options & function_options::expl_derivative_func(expl_derivative_funcp_2 d)
+{
+	test_and_set_nparams(2);
+	expl_derivative_f = expl_derivative_funcp(d);
+	return *this;
+}
+function_options & function_options::expl_derivative_func(expl_derivative_funcp_3 d)
+{
+	test_and_set_nparams(3);
+	expl_derivative_f = expl_derivative_funcp(d);
+	return *this;
+}
+
 function_options & function_options::power_func(power_funcp_1 d)
 {
 	test_and_set_nparams(1);
@@ -533,6 +554,16 @@ function::function(unsigned ser, const ex & param1, const ex & param2, const ex
 {
 	tinfo_key = &function::tinfo_static;
 }
+function::function(unsigned ser, const ex & param1, const ex & param2, const ex & param3, const ex & param4)
+	: exprseq(param1, param2, param3, param4), serial(ser)
+{
+	tinfo_key = &function::tinfo_static;
+}
+function::function(unsigned ser, const ex & param1, const ex & param2, const ex & param3, const ex & param4, const ex & param5)
+	: exprseq(param1, param2, param3, param4, param5), serial(ser)
+{
+	tinfo_key = &function::tinfo_static;
+}
 function::function(unsigned ser, const ex & param1, const ex & param2, const ex & param3, const ex & param4, const ex & param5, const ex & param6)
 	: exprseq(param1, param2, param3, param4, param5, param6), serial(ser)
 {
@@ -881,6 +912,9 @@ ex function::eval(int level) const
 	case 3:
 		eval_result = ((eval_funcp_3)(opt.eval_f))(seq[1-1], seq[2-1], seq[3-1]);
 		break;
+	case 6:
+		eval_result = ((eval_funcp_6)(opt.eval_f))(seq[1-1], seq[2-1], seq[3-1], seq[4-1], seq[5-1], seq[6-1]);
+		break;
 
 		// end of generated lines
 	default:
@@ -910,7 +944,23 @@ ex function::evalf(int level, PyObject* kwds) const
 			eseq.push_back(elem.evalf(level, kwds));
 	}
 
-	if (opt.evalf_f==nullptr) {
+	if (opt.evalf_f == nullptr) {
+                if (opt.nparams == 1 and is_exactly_a<numeric>(eseq[1-1])) {
+                        const numeric& n = ex_to<numeric>(eseq[1-1]);
+                        try {
+                                return numeric::try_py_method(n, get_name());
+                        }
+                        catch (std::logic_error) {
+                                try {
+                                        ex e = numeric::try_py_method(ex_to<numeric>(n.evalf()),
+                                                           get_name());
+                                        if (not is_exactly_a<numeric>(e))
+                                                throw std::runtime_error("can't happen in function::evalf");
+                                        return numeric::to_dict_parent(ex_to<numeric>(e), kwds);
+                                }
+                                catch (std::logic_error) {}
+                        }
+                }
 		return function(serial,eseq).hold();
 	}
 	current_serial = serial;
@@ -943,6 +993,8 @@ ex function::evalf(int level, PyObject* kwds) const
 		return ((evalf_funcp_2)(opt.evalf_f))(eseq[1-1], eseq[2-1], kwds);
 	case 3:
 		return ((evalf_funcp_3)(opt.evalf_f))(eseq[1-1], eseq[2-1], eseq[3-1], kwds);
+	case 6:
+		return ((evalf_funcp_6)(opt.evalf_f))(eseq[1-1], eseq[2-1], eseq[3-1], eseq[4-1], eseq[5-1], eseq[6-1], kwds);
 
 		// end of generated lines
 	}
@@ -951,7 +1003,7 @@ ex function::evalf(int level, PyObject* kwds) const
 
 long function::calchash() const
 {
-	long v = golden_ratio_hash(golden_ratio_hash((p_int)tinfo()) ^ serial);
+	long v = golden_ratio_hash(golden_ratio_hash((intptr_t)tinfo()) ^ serial);
 	for (size_t i=0; i<nops(); i++) {
 		v = rotate_left(v);
 		v ^= this->op(i).gethash();
@@ -1230,6 +1282,11 @@ ex function::derivative(const symbol & s) const
 	GINAC_ASSERT(serial<registered_functions().size());
 	const function_options &opt = registered_functions()[serial];
 
+        try {
+                // Explicit derivation
+                return expl_derivative(s);
+        } catch (...) {}
+
 	// Check if we need to apply chain rule
 	if (!opt.apply_chain_rule) {
 		if (opt.derivative_f == nullptr)
@@ -1429,12 +1486,38 @@ ex function::pderivative(unsigned diff_param) const // partial differentiation
 		return ((derivative_funcp_2)(opt.derivative_f))(seq[1-1], seq[2-1],diff_param);
 	case 3:
 		return ((derivative_funcp_3)(opt.derivative_f))(seq[1-1], seq[2-1], seq[3-1],diff_param);
+	case 6:
+		return ((derivative_funcp_6)(opt.derivative_f))(seq[1-1], seq[2-1], seq[3-1], seq[4-1], seq[5-1], seq[6-1], diff_param);
 
 		// end of generated lines
 	}
 	throw(std::logic_error("function::pderivative(): no diff function defined"));
 }
 
+ex function::expl_derivative(const symbol & s) const // explicit differentiation
+{
+	GINAC_ASSERT(serial<registered_functions().size());
+	const function_options &opt = registered_functions()[serial];
+
+        if (opt.expl_derivative_f) {
+		// Invoke the defined explicit derivative function.
+		current_serial = serial;
+		if (opt.expl_derivative_use_exvector_args)
+			return ((expl_derivative_funcp_exvector)(opt.expl_derivative_f))(seq, s);
+		switch (opt.nparams) {
+			// the following lines have been generated for max. 14 parameters
+			case 1:
+				return ((expl_derivative_funcp_1)(opt.expl_derivative_f))(seq[0], s);
+			case 2:
+				return ((expl_derivative_funcp_2)(opt.expl_derivative_f))(seq[0], seq[1], s);
+			case 3:
+				return ((expl_derivative_funcp_3)(opt.expl_derivative_f))(seq[0], seq[1], seq[2], s);
+		}
+	}
+	// There is no fallback for explicit derivative.
+	throw(std::logic_error("function::expl_derivative(): explicit derivation is called, but no such function defined"));
+}
+
 ex function::power(const ex & power_param) const // power of function
 {
 	GINAC_ASSERT(serial<registered_functions().size());
@@ -1578,91 +1661,6 @@ void function::set_domain(unsigned d)
         }
 }
 
-bool function::info(unsigned inf) const
-{
-        if (serial == exp_SERIAL::serial)
-        {
-                const ex& arg = op(0);
-                switch (inf) {
-                case info_flags::nonzero:
-                        return true;
-                case info_flags::real:
-                case info_flags::positive:
-                        return arg.info(info_flags::real);
-                }
-                return false;
-        }
-        else if (serial == log_SERIAL::serial) {
-                const ex& arg = op(0);
-                switch (inf) {
-                case info_flags::real:
-                        return arg.info(info_flags::positive);
-                case info_flags::positive:
-                        return arg.info(info_flags::real) and
-                                is_exactly_a<numeric>(arg) and
-                                ex_to<numeric>(arg) > *_num1_p;
-                case info_flags::negative:
-                        return arg.info(info_flags::real) and
-                                is_exactly_a<numeric>(arg) and
-                                arg.info(info_flags::positive) and
-                                ex_to<numeric>(arg) < *_num1_p;
-                }
-                return false;
-        }
-        else if (serial == abs_SERIAL::serial) {
-                switch (inf) {
-                case info_flags::real:
-                case info_flags::nonnegative:
-                        return true;
-                case info_flags::nonzero:
-                        return op(0).info(info_flags::nonzero);
-                }
-                return false;
-        }
-        else if (serial == real_part_function_SERIAL::serial) {
-                switch (inf) {
-                case info_flags::real:
-                case info_flags::nonnegative:
-                        return true;
-                case info_flags::nonzero:
-                        return op(0).info(info_flags::nonzero);
-                }
-                return false;
-        }
-        else if (serial == imag_part_function_SERIAL::serial) {
-                switch (inf) {
-                case info_flags::real:
-                case info_flags::nonnegative:
-                        return true;
-                case info_flags::nonzero:
-                        return op(0).info(info_flags::nonzero);
-                }
-                return false;
-        }
-        else if (serial == binomial_SERIAL::serial) {
-                const ex& arg = op(0);
-                switch (inf) {
-                case static_cast<unsigned int>((info_flags::real != 0u) or (info_flags::nonnegative != 0u)):
-                        return arg.info(inf);
-                case info_flags::integer:
-                        return arg.info(inf) and op(1).info(inf);
-                }
-                return false;
-        }
-        else if (serial == factorial_SERIAL::serial) {
-                const ex& arg = op(0);
-                switch (inf) {
-                case static_cast<unsigned int>((info_flags::real != 0u) or (info_flags::nonnegative != 0u)):
-                        return arg.info(inf);
-                case info_flags::integer:
-                        return arg.info(inf);
-                }
-                return false;
-        }
-
-	return iflags.get(inf);
-}
-
 bool has_function(const ex & x)
 {
 	if (is_exactly_a<function>(x))
