@@ -35,7 +35,6 @@
 #include "constant.h"
 #include "infinity.h"
 #include "fail.h"
-#include "integral.h"
 #include "lst.h"
 #include "matrix.h"
 #include "numeric.h"
@@ -50,6 +49,8 @@
 #include "expairseq.h"
 #include "add.h"
 #include "mul.h"
+#include "upoly.h"
+#include "mpoly.h"
 
 #include "exprseq.h"
 #include "function.h"
@@ -60,6 +61,7 @@
 #include "operators.h"
 
 #include "assume.h"
+#include "sum.h"
 
 #include "idx.h"
 #include "indexed.h"
