@@ -70,7 +70,7 @@ static ex lgamma_eval(const ex & x)
 				return Infinity;
 				//throw (pole_error("lgamma_eval(): logarithmic pole",0));
 		}
-		if (!ex_to<numeric>(x).is_crational())
+		if (!ex_to<numeric>(x).is_exact())
 			return lgamma(ex_to<numeric>(x));
 	}
 	
@@ -176,16 +176,16 @@ static ex tgamma_eval(const ex & x)
 			// trap positive x==(n+1/2)
 			// tgamma(n+1/2) -> Pi^(1/2)*(1*3*..*(2*n-1))/(2^n)
 			if (two_x.is_positive()) {
-				const numeric n = ex_to<numeric>(x).sub(*_num1_2_p);
-				return (doublefactorial(n.mul(*_num2_p).sub(*_num1_p)).div(pow(*_num2_p,n))) * sqrt(Pi);
+				const numeric n = ex_to<numeric>(x).sub(*_num1_2_p).to_bigint();
+				return (doublefactorial(n.mul(*_num2_p).sub(*_num1_p)).div(_num2_p->pow_intexp(n))) * sqrt(Pi);
 			} else {
 				// trap negative x==(-n+1/2)
 				// tgamma(-n+1/2) -> Pi^(1/2)*(-2)^n/(1*3*..*(2*n-1))
-				const numeric n = abs(ex_to<numeric>(x).sub(*_num1_2_p));
-				return (pow(*_num_2_p, n).div(doublefactorial(n.mul(*_num2_p).sub(*_num1_p))))*sqrt(Pi);
+				const numeric n = abs(ex_to<numeric>(x).sub(*_num1_2_p).to_bigint());
+				return _num_2_p->pow_intexp(n).div(doublefactorial(n.mul(*_num2_p).sub(*_num1_p)))*sqrt(Pi);
 			}
 		}
-		if (!ex_to<numeric>(x).is_crational())
+		if (!ex_to<numeric>(x).is_exact())
 			return tgamma_evalf(ex_to<numeric>(x), nullptr);
 	}
 	
@@ -296,7 +296,7 @@ static ex beta_eval(const ex & x, const ex & y)
 		    (nx+ny).is_integer() &&
 		   !(nx+ny).is_positive())
 			 return _ex0;
-		if (not nx.is_crational() or not ny.is_crational())
+		if (not nx.is_exact() or not ny.is_exact())
 			return beta_evalf(x, y, nullptr);
 	}
 	
@@ -415,11 +415,11 @@ static ex psi1_eval(const ex & x)
 				// where r == ((-1/2)^(-1) + ... + (-m-1/2)^(-1))
 				numeric recur = 0;
 				for (numeric p = nx; p<0; ++p)
-					recur -= pow(p, *_num_1_p);
+					recur -= p.inverse();
 				return recur+psi(_ex1_2);
 			}
 		}
-		if (not nx.is_crational())
+		if (not nx.is_exact())
 			return psi(nx);
 	}
 	
@@ -503,7 +503,7 @@ static ex psi2_eval(const ex & n, const ex & x)
 			// integer case 
 			if (nx.is_equal(*_num1_p))
 				// use psi(n,1) == (-)^(n+1) * n! * zeta(n+1)
-				return pow(*_num_1_p,nn+(*_num1_p))*factorial(nn)*zeta(ex(nn+(*_num1_p)));
+				return _num_1_p->pow_intexp(nn+(*_num1_p))*factorial(nn)*zeta(ex(nn+(*_num1_p)));
 			if (nx.is_positive()) {
 				// use the recurrence relation
 				//   psi(n,m) == psi(n,m+1) - (-)^n * n! / m^(n+1)
@@ -512,8 +512,8 @@ static ex psi2_eval(const ex & n, const ex & x)
 				// where r == (-)^n * n! * (1^(-n-1) + ... + (m-1)^(-n-1))
 				numeric recur = 0;
 				for (numeric p = 1; p<nx; ++p)
-					recur += pow(p, -nn+(*_num_1_p));
-				recur *= factorial(nn)*pow((*_num_1_p), nn);
+					recur += p.pow_intexp(-nn+(*_num_1_p));
+				recur *= factorial(nn)*(_num_1_p->pow_intexp(nn));
 				return recur+psi(n,_ex1);
 			} else {
 				// for non-positive integers there is a pole:
@@ -524,13 +524,13 @@ static ex psi2_eval(const ex & n, const ex & x)
 			// half integer case
 			if (nx.is_equal(*_num1_2_p))
 				// use psi(n,1/2) == (-)^(n+1) * n! * (2^(n+1)-1) * zeta(n+1)
-				return pow(*_num_1_p,nn+(*_num1_p))*factorial(nn)*(pow(*_num2_p,nn+(*_num1_p)) + (*_num_1_p))*zeta(ex(nn+(*_num1_p)));
+				return _num_1_p->pow_intexp(nn+(*_num1_p))*factorial(nn)*(_num2_p->pow_intexp(nn+(*_num1_p)) + (*_num_1_p))*zeta(ex(nn+(*_num1_p)));
 			if (nx.is_positive()) {
 				const numeric m = nx - (*_num1_2_p);
 				// use the multiplication formula
 				//   psi(n,2*m) == (psi(n,m) + psi(n,m+1/2)) / 2^(n+1)
 				// to revert to positive integer case
-				return psi(n,(*_num2_p)*m)*pow((*_num2_p),nn+(*_num1_p))-psi(n,m);
+				return psi(n,(*_num2_p)*m)*_num2_p->pow_intexp(nn+(*_num1_p))-psi(n,m);
 			} else {
 				// use the recurrence relation
 				//   psi(n,-m-1/2) == psi(n,-m-1/2+1) - (-)^n * n! / (-m-1/2)^(n+1)
@@ -539,8 +539,8 @@ static ex psi2_eval(const ex & n, const ex & x)
 				// where r == (-)^(n+1) * n! * ((-1/2)^(-n-1) + ... + (-m-1/2)^(-n-1))
 				numeric recur = 0;
 				for (numeric p = nx; p<0; ++p)
-					recur += pow(p, -nn+(*_num_1_p));
-				recur *= factorial(nn)*pow(*_num_1_p, nn+(*_num_1_p));
+					recur += p.pow_intexp(-nn+(*_num_1_p));
+				recur *= factorial(nn)*_num_1_p->pow_intexp(nn+(*_num_1_p));
 				return recur+psi(n,_ex1_2);
 			}
 		}
