@@ -55,14 +55,6 @@ namespace GiNaC {
 // hyperbolic sine (trigonometric function)
 //////////
 
-static ex sinh_evalf(const ex & x, PyObject* parent)
-{
-	if (is_exactly_a<numeric>(x))
-		return sinh(ex_to<numeric>(x));
-	
-	return sinh(x).hold();
-}
-
 static ex sinh_eval(const ex & x)
 {
 	if (is_exactly_a<numeric>(x)) {
@@ -72,7 +64,7 @@ static ex sinh_eval(const ex & x)
 			return _ex0;        
 
 		// sinh(float) -> float
-		if (!x.info(info_flags::crational))
+		if (x.info(info_flags::inexact))
 			return sinh(ex_to<numeric>(x));
 
 		// sinh() is odd
@@ -138,7 +130,6 @@ static ex sinh_conjugate(const ex & x)
 }
 
 REGISTER_FUNCTION(sinh, eval_func(sinh_eval).
-                        evalf_func(sinh_evalf).
                         derivative_func(sinh_deriv).
                         real_part_func(sinh_real_part).
                         imag_part_func(sinh_imag_part).
@@ -166,7 +157,7 @@ static ex cosh_eval(const ex & x)
 			return _ex1;
 
 		// cosh(float) -> float
-		if (!x.info(info_flags::crational))
+		if (x.info(info_flags::inexact))
 			return cosh(ex_to<numeric>(x));
 
 		// cosh() is even
@@ -243,14 +234,6 @@ REGISTER_FUNCTION(cosh, eval_func(cosh_eval).
 // hyperbolic tangent (trigonometric function)
 //////////
 
-static ex tanh_evalf(const ex & x, PyObject* parent)
-{
-	if (is_exactly_a<numeric>(x))
-		return tanh(ex_to<numeric>(x));
-	
-	return tanh(x).hold();
-}
-
 static ex tanh_eval(const ex & x)
 {
 	if (is_exactly_a<numeric>(x)) {
@@ -260,7 +243,7 @@ static ex tanh_eval(const ex & x)
 			return _ex0;
 
 		// tanh(float) -> float
-		if (!x.info(info_flags::crational))
+		if (x.info(info_flags::inexact))
 			return tanh(ex_to<numeric>(x));
 
 		// tanh() is odd
@@ -350,7 +333,6 @@ static ex tanh_conjugate(const ex & x)
 }
 
 REGISTER_FUNCTION(tanh, eval_func(tanh_eval).
-                        evalf_func(tanh_evalf).
                         derivative_func(tanh_deriv).
                         series_func(tanh_series).
                         real_part_func(tanh_real_part).
@@ -362,14 +344,6 @@ REGISTER_FUNCTION(tanh, eval_func(tanh_eval).
 // hyperbolic cotangent (trigonometric function)
 //////////
 
-static ex coth_evalf(const ex & x, PyObject* parent)
-{
-	if (is_exactly_a<numeric>(x))
-		return tanh(ex_to<numeric>(x)).inverse();
-
-	return tanh(x).hold();
-}
-
 static ex coth_eval(const ex & x)
 {
 	if (is_exactly_a<numeric>(x)) {
@@ -379,7 +353,7 @@ static ex coth_eval(const ex & x)
 			return UnsignedInfinity;
 
 		// coth(float) -> float
-		if (!x.info(info_flags::crational))
+		if (x.info(info_flags::inexact))
 			return tanh(ex_to<numeric>(x)).inverse();
 
 		// coth() is odd
@@ -478,7 +452,6 @@ static ex coth_conjugate(const ex & x)
 }
 
 REGISTER_FUNCTION(coth, eval_func(coth_eval).
-                        evalf_func(coth_evalf).
                         derivative_func(coth_deriv).
                         series_func(coth_series).
                         real_part_func(coth_real_part).
@@ -490,14 +463,6 @@ REGISTER_FUNCTION(coth, eval_func(coth_eval).
 // hyperbolic secant (trigonometric function)
 //////////
 
-static ex sech_evalf(const ex & x, PyObject* parent)
-{
-	if (is_exactly_a<numeric>(x))
-		return cosh(ex_to<numeric>(x)).inverse();
-
-	return sech(x).hold();
-}
-
 static ex sech_eval(const ex & x)
 {
 	if (is_exactly_a<numeric>(x)) {
@@ -507,7 +472,7 @@ static ex sech_eval(const ex & x)
 			return _ex1;
 
 		// sech(float) -> float
-		if (!x.info(info_flags::crational))
+		if (x.info(info_flags::inexact))
 			return cosh(ex_to<numeric>(x)).inverse();
 
 		// sech() is even
@@ -593,7 +558,6 @@ static ex sech_conjugate(const ex & x)
 }
 
 REGISTER_FUNCTION(sech, eval_func(sech_eval).
-                        evalf_func(sech_evalf).
                         derivative_func(sech_deriv).
                         series_func(sech_series).
                         real_part_func(sech_real_part).
@@ -605,14 +569,6 @@ REGISTER_FUNCTION(sech, eval_func(sech_eval).
 // hyperbolic secant (trigonometric function)
 //////////
 
-static ex csch_evalf(const ex & x, PyObject* parent)
-{
-	if (is_exactly_a<numeric>(x))
-		return sinh(ex_to<numeric>(x)).inverse();
-
-	return csch(x).hold();
-}
-
 static ex csch_eval(const ex & x)
 {
 	if (is_exactly_a<numeric>(x)) {
@@ -622,7 +578,7 @@ static ex csch_eval(const ex & x)
 			return UnsignedInfinity;
 
 		// csch(float) -> float
-		if (!x.info(info_flags::crational))
+		if (x.info(info_flags::inexact))
 			return sinh(ex_to<numeric>(x)).inverse();
 
 		// csch() is odd
@@ -708,7 +664,6 @@ static ex csch_conjugate(const ex & x)
 }
 
 REGISTER_FUNCTION(csch, eval_func(csch_eval).
-                        evalf_func(csch_evalf).
                         derivative_func(csch_deriv).
                         series_func(csch_series).
                         real_part_func(csch_real_part).
@@ -720,14 +675,6 @@ REGISTER_FUNCTION(csch, eval_func(csch_eval).
 // inverse hyperbolic sine (trigonometric function)
 //////////
 
-static ex asinh_evalf(const ex & x, PyObject* parent)
-{
-	if (is_exactly_a<numeric>(x))
-		return asinh(ex_to<numeric>(x));
-	
-	return asinh(x).hold();
-}
-
 static ex asinh_eval(const ex & x)
 {
 	if (is_exactly_a<numeric>(x)) {
@@ -737,7 +684,7 @@ static ex asinh_eval(const ex & x)
 			return _ex0;
 
 		// asinh(float) -> float
-		if (!x.info(info_flags::crational))
+		if (x.info(info_flags::inexact))
 			return asinh(ex_to<numeric>(x));
 
 		// asinh() is odd
@@ -780,7 +727,6 @@ static ex asinh_conjugate(const ex & x)
 }
 
 REGISTER_FUNCTION(asinh, eval_func(asinh_eval).
-                         evalf_func(asinh_evalf).
                          derivative_func(asinh_deriv).
                          conjugate_func(asinh_conjugate).
 			 set_name("arcsinh"));
@@ -789,14 +735,6 @@ REGISTER_FUNCTION(asinh, eval_func(asinh_eval).
 // inverse hyperbolic cosine (trigonometric function)
 //////////
 
-static ex acosh_evalf(const ex & x, PyObject* parent)
-{
-	if (is_exactly_a<numeric>(x))
-		return acosh(ex_to<numeric>(x));
-	
-	return acosh(x).hold();
-}
-
 static ex acosh_eval(const ex & x)
 {
 	if (is_exactly_a<numeric>(x)) {
@@ -814,7 +752,7 @@ static ex acosh_eval(const ex & x)
 			return Pi*I;
 
 		// acosh(float) -> float
-		if (!x.info(info_flags::crational))
+		if (x.info(info_flags::inexact))
 			return acosh(ex_to<numeric>(x));
 
 		// acosh(-x) -> Pi*I-acosh(x)
@@ -852,7 +790,6 @@ static ex acosh_conjugate(const ex & x)
 }
 
 REGISTER_FUNCTION(acosh, eval_func(acosh_eval).
-                         evalf_func(acosh_evalf).
                          derivative_func(acosh_deriv).
                          conjugate_func(acosh_conjugate).
 			 set_name("arccosh"));
@@ -861,14 +798,6 @@ REGISTER_FUNCTION(acosh, eval_func(acosh_eval).
 // inverse hyperbolic tangent (trigonometric function)
 //////////
 
-static ex atanh_evalf(const ex & x, PyObject* parent)
-{
-	if (is_exactly_a<numeric>(x))
-		return atanh(ex_to<numeric>(x));
-	
-	return atanh(x).hold();
-}
-
 static ex atanh_eval(const ex & x)
 {
 	if (is_exactly_a<numeric>(x)) {
@@ -891,7 +820,7 @@ static ex atanh_eval(const ex & x)
 			return NegInfinity;
 
 		// atanh(float) -> float
-		if (!x.info(info_flags::crational))
+		if (x.info(info_flags::inexact))
 			return atanh(ex_to<numeric>(x));
 
 		// atanh() is odd
@@ -977,7 +906,6 @@ static ex atanh_conjugate(const ex & x)
 }
 
 REGISTER_FUNCTION(atanh, eval_func(atanh_eval).
-                         evalf_func(atanh_evalf).
                          derivative_func(atanh_deriv).
                          series_func(atanh_series).
                          conjugate_func(atanh_conjugate).
@@ -987,14 +915,6 @@ REGISTER_FUNCTION(atanh, eval_func(atanh_eval).
 // inverse hyperbolic cotangent (trigonometric function)
 //////////
 
-static ex acoth_evalf(const ex & x, PyObject* parent)
-{
-        if (is_exactly_a<numeric>(x))
-                return atanh(ex_to<numeric>(x).inverse());
-
-        return acoth(x).hold();
-}
-
 static ex acoth_eval(const ex & x)
 {
         if (is_exactly_a<numeric>(x)) {
@@ -1005,7 +925,7 @@ static ex acoth_eval(const ex & x)
                 if (x.is_equal(_ex_1))
                         return NegInfinity;
                 //acoth(float) -> float 
-                if (!x.info(info_flags::crational))
+                if (x.info(info_flags::inexact))
                         return atanh(ex_to<numeric>(x).inverse());
                 // acoth() is odd
                 if (x.info(info_flags::negative))
@@ -1053,7 +973,6 @@ static ex acoth_conjugate(const ex & x)
 }
 
 REGISTER_FUNCTION(acoth, eval_func(acoth_eval).
-                         evalf_func(acoth_evalf).
                          derivative_func(acoth_deriv).
                          conjugate_func(acoth_conjugate).
                          set_name("arccoth"));
@@ -1062,14 +981,6 @@ REGISTER_FUNCTION(acoth, eval_func(acoth_eval).
 // inverse hyperbolic Cosecant (trigonometric function)
 //////////
 
-static ex acsch_evalf(const ex & x, PyObject* parent)
-{
-        if (is_exactly_a<numeric>(x))
-                return asinh(ex_to<numeric>(x).inverse());
-
-        return acsch(x).hold();
-}
-
 static ex acsch_eval(const ex & x)
 {
         if (is_exactly_a<numeric>(x)) {
@@ -1077,7 +988,7 @@ static ex acsch_eval(const ex & x)
                 if (x.is_zero())
                         return Infinity;
                 //acsch(float) -> float 
-                if (!x.info(info_flags::crational))
+                if (x.info(info_flags::inexact))
                         return asinh(ex_to<numeric>(x).inverse());
                 // acsch(-x) -> acsch(-x)
                 if (x.info(info_flags::negative))
@@ -1120,7 +1031,6 @@ static ex acsch_conjugate(const ex & x)
 }
 
 REGISTER_FUNCTION(acsch, eval_func(acsch_eval).
-                         evalf_func(acsch_evalf).
                          derivative_func(acsch_deriv).
                          conjugate_func(acsch_conjugate).
                          set_name("arccsch"));
@@ -1128,14 +1038,6 @@ REGISTER_FUNCTION(acsch, eval_func(acsch_eval).
 // inverse hyperbolic Secant (trigonometric function)
 //////////
 
-static ex asech_evalf(const ex & x, PyObject* parent)
-{
-        if (is_exactly_a<numeric>(x))
-                return acosh(ex_to<numeric>(x).inverse());
-
-        return asech(x).hold();
-}
-
 static ex asech_eval(const ex & x)
 {
         if (is_exactly_a<numeric>(x)) {
@@ -1149,7 +1051,7 @@ static ex asech_eval(const ex & x)
                 if (x.is_equal(_ex_1))
                         return Pi*I;
                 //asech(float) -> float 
-                if (!x.info(info_flags::crational))
+                if (x.info(info_flags::inexact))
                         return acosh(ex_to<numeric>(x).inverse());
                 // asech(-x) -> Pi*I-asech(-x)
                 if (x.info(info_flags::negative))
@@ -1190,7 +1092,6 @@ static ex asech_conjugate(const ex & x)
 }
 
 REGISTER_FUNCTION(asech, eval_func(asech_eval).
-                         evalf_func(asech_evalf).
                          derivative_func(asech_deriv).
                          conjugate_func(asech_conjugate).
                          set_name("arcsech"));
