@@ -52,14 +52,6 @@ static bool has_pi(const ex & the_ex) {
 // sine (trigonometric function)
 //////////
 
-static ex sin_evalf(const ex & x, PyObject* parent)
-{
-	if (is_exactly_a<numeric>(x))
-		return sin(ex_to<numeric>(x));
-	
-	return sin(x).hold();
-}
-
 static ex sin_eval(const ex & x)
 {
 	// sin(oo) -> error
@@ -230,7 +222,7 @@ static ex sin_eval(const ex & x)
 	}
 
 	// sin(float) -> float
-        if (is_exactly_a<numeric>(x_red) && !x_red.info(info_flags::crational))
+        if (is_exactly_a<numeric>(x_red) && x_red.info(info_flags::inexact))
 		return sin(ex_to<numeric>(x_red));
 
 	return sin(x_red).hold();
@@ -261,7 +253,6 @@ static ex sin_conjugate(const ex & x)
 }
 
 REGISTER_FUNCTION(sin, eval_func(sin_eval).
-                       evalf_func(sin_evalf).
                        derivative_func(sin_deriv).
                        real_part_func(sin_real_part).
                        imag_part_func(sin_imag_part).
@@ -272,14 +263,6 @@ REGISTER_FUNCTION(sin, eval_func(sin_eval).
 // cosine (trigonometric function)
 //////////
 
-static ex cos_evalf(const ex & x, PyObject* parent)
-{
-	if (is_exactly_a<numeric>(x))
-		return cos(ex_to<numeric>(x));
-	
-	return cos(x).hold();
-}
-
 static ex cos_eval(const ex & x)
 {
 	// cos(oo) -> error
@@ -445,7 +428,7 @@ static ex cos_eval(const ex & x)
 	}
 
 	// cos(float) -> float
-	if (is_exactly_a<numeric>(x_red) && !x_red.info(info_flags::crational))
+	if (is_exactly_a<numeric>(x_red) && x_red.info(info_flags::inexact))
 		return cos(ex_to<numeric>(x_red));
 	
 	return cos(x_red).hold();
@@ -476,7 +459,6 @@ static ex cos_conjugate(const ex & x)
 }
 
 REGISTER_FUNCTION(cos, eval_func(cos_eval).
-                       evalf_func(cos_evalf).
                        derivative_func(cos_deriv).
                        real_part_func(cos_real_part).
                        imag_part_func(cos_imag_part).
@@ -487,14 +469,6 @@ REGISTER_FUNCTION(cos, eval_func(cos_eval).
 // tangent (trigonometric function)
 //////////
 
-static ex tan_evalf(const ex & x, PyObject* parent)
-{
-	if (is_exactly_a<numeric>(x))
-		return tan(ex_to<numeric>(x));
-	
-	return tan(x).hold();
-}
-
 static ex tan_eval(const ex & x)
 {
 	// tan(oo) -> error
@@ -664,7 +638,7 @@ static ex tan_eval(const ex & x)
 	}
 
 	// tan(float) -> float
-	if (is_exactly_a<numeric>(x_red) && !x_red.info(info_flags::crational)) {
+	if (is_exactly_a<numeric>(x_red) && x_red.info(info_flags::inexact)) {
 		return tan(ex_to<numeric>(x_red));
 	}
 	
@@ -716,7 +690,6 @@ static ex tan_conjugate(const ex & x)
 }
 
 REGISTER_FUNCTION(tan, eval_func(tan_eval).
-                       evalf_func(tan_evalf).
                        derivative_func(tan_deriv).
                        series_func(tan_series).
                        real_part_func(tan_real_part).
@@ -728,17 +701,6 @@ REGISTER_FUNCTION(tan, eval_func(tan_eval).
 // cotangent (trigonometric function)
 //////////
 
-static ex cot_evalf(const ex & x, PyObject* parent)
-{
-	if (is_exactly_a<numeric>(x)) {
-                if (ex_to<numeric>(x).is_zero())
-                        return UnsignedInfinity;
-		return tan(ex_to<numeric>(x)).inverse();
-        }
-
-	return cot(x).hold();
-}
-
 static ex cot_eval(const ex & x)
 {
 	if (is_exactly_a<function>(x)) {
@@ -776,7 +738,7 @@ static ex cot_eval(const ex & x)
 	}
 
 	// cot(float) -> float
-	if (is_exactly_a<numeric>(x) && !x.info(info_flags::crational)) {
+	if (is_exactly_a<numeric>(x) && x.info(info_flags::inexact)) {
                 if (ex_to<numeric>(x).is_zero())
                         return UnsignedInfinity;
 		return tan(ex_to<numeric>(x)).inverse();
@@ -860,7 +822,6 @@ static ex cot_conjugate(const ex & x)
 }
 
 REGISTER_FUNCTION(cot, eval_func(cot_eval).
-                       evalf_func(cot_evalf).
                        derivative_func(cot_deriv).
                        series_func(cot_series).
                        real_part_func(cot_real_part).
@@ -872,14 +833,6 @@ REGISTER_FUNCTION(cot, eval_func(cot_eval).
 // secant (trigonometric function)
 //////////
 
-static ex sec_evalf(const ex & x, PyObject* parent)
-{
-	if (is_exactly_a<numeric>(x))
-		return cos(ex_to<numeric>(x)).inverse();
-
-	return sec(x).hold();
-}
-
 static ex sec_eval(const ex & x)
 {
 	if (is_exactly_a<function>(x)) {
@@ -922,7 +875,7 @@ static ex sec_eval(const ex & x)
 		return sec(-x);
 
 	// sec(float) -> float
-	if (is_exactly_a<numeric>(x) && !x.info(info_flags::crational)) {
+	if (is_exactly_a<numeric>(x) && x.info(info_flags::inexact)) {
 		return cos(ex_to<numeric>(x)).inverse();
 	}
 
@@ -981,7 +934,6 @@ static ex sec_conjugate(const ex & x)
 }
 
 REGISTER_FUNCTION(sec, eval_func(sec_eval).
-                       evalf_func(sec_evalf).
                        derivative_func(sec_deriv).
                        series_func(sec_series).
                        real_part_func(sec_real_part).
@@ -993,17 +945,6 @@ REGISTER_FUNCTION(sec, eval_func(sec_eval).
 // cosecant (trigonometric function)
 //////////
 
-static ex csc_evalf(const ex & x, PyObject* parent)
-{
-	if (is_exactly_a<numeric>(x)) {
-                if (ex_to<numeric>(x).is_zero())
-                        return UnsignedInfinity;
-		return sin(ex_to<numeric>(x)).inverse();
-        }
-
-	return csc(x).hold();
-}
-
 static ex csc_eval(const ex & x)
 {
 
@@ -1043,7 +984,7 @@ static ex csc_eval(const ex & x)
 	}
 
 	// csc(float) -> float
-	if (is_exactly_a<numeric>(x) && !x.info(info_flags::crational)) {
+	if (is_exactly_a<numeric>(x) && x.info(info_flags::inexact)) {
                 if (ex_to<numeric>(x).is_zero())
                         return UnsignedInfinity;
 		return sin(ex_to<numeric>(x)).inverse();
@@ -1104,7 +1045,6 @@ static ex csc_conjugate(const ex & x)
 }
 
 REGISTER_FUNCTION(csc, eval_func(csc_eval).
-                       evalf_func(csc_evalf).
                        derivative_func(csc_deriv).
                        series_func(csc_series).
                        real_part_func(csc_real_part).
@@ -1116,14 +1056,6 @@ REGISTER_FUNCTION(csc, eval_func(csc_eval).
 // inverse sine (arc sine)
 //////////
 
-static ex asin_evalf(const ex & x, PyObject* parent)
-{
-	if (is_exactly_a<numeric>(x))
-		return asin(ex_to<numeric>(x));
-	
-	return asin(x).hold();
-}
-
 static ex asin_eval(const ex & x)
 {
 	if (is_exactly_a<numeric>(x)) {
@@ -1148,9 +1080,15 @@ static ex asin_eval(const ex & x)
 		if (x.is_equal(_ex_1))
 			return _ex_1_2*Pi;
 
-		// asin(float) -> float
-		if (!x.info(info_flags::crational))
-			return asin(ex_to<numeric>(x));
+		if (x.info(info_flags::inexact)) {
+                        const numeric& num = ex_to<numeric>(x);
+                        if (num.is_real()
+                            and (num < *_num_1_p or num > *_num1_p))
+                                return NaN;
+                        else
+        		        // asin(float) -> float
+	        		return asin(ex_to<numeric>(x));
+                }
 
 		// asin() is odd
 		if (x.info(info_flags::negative))
@@ -1188,7 +1126,6 @@ static ex asin_conjugate(const ex & x)
 }
 
 REGISTER_FUNCTION(asin, eval_func(asin_eval).
-                        evalf_func(asin_evalf).
                         derivative_func(asin_deriv).
                         conjugate_func(asin_conjugate).
 			set_name("arcsin", "\\arcsin"));
@@ -1197,14 +1134,6 @@ REGISTER_FUNCTION(asin, eval_func(asin_eval).
 // inverse cosine (arc cosine)
 //////////
 
-static ex acos_evalf(const ex & x, PyObject* parent)
-{
-	if (is_exactly_a<numeric>(x))
-		return acos(ex_to<numeric>(x));
-	
-	return acos(x).hold();
-}
-
 static ex acos_eval(const ex & x)
 {
 	if (is_exactly_a<numeric>(x)) {
@@ -1229,9 +1158,15 @@ static ex acos_eval(const ex & x)
 		if (x.is_equal(_ex_1))
 			return Pi;
 
-		// acos(float) -> float
-		if (!x.info(info_flags::crational))
-			return acos(ex_to<numeric>(x));
+		if (x.info(info_flags::inexact)) {
+                        const numeric& num = ex_to<numeric>(x);
+                        if (num.is_real()
+                            and (num < *_num_1_p or num > *_num1_p))
+                                return NaN;
+                        else
+        		        // acos(float) -> float
+	        		return acos(ex_to<numeric>(x));
+                }
 
 		// acos(-x) -> Pi-acos(x)
 		if (x.info(info_flags::negative))
@@ -1269,7 +1204,6 @@ static ex acos_conjugate(const ex & x)
 
  
 REGISTER_FUNCTION(acos, eval_func(acos_eval).
-                        evalf_func(acos_evalf).
                         derivative_func(acos_deriv).
                         conjugate_func(acos_conjugate).
 			set_name("arccos", "\\arccos"));
@@ -1278,14 +1212,6 @@ REGISTER_FUNCTION(acos, eval_func(acos_eval).
 // inverse tangent (arc tangent)
 //////////
 
-static ex atan_evalf(const ex & x, PyObject* parent)
-{
-	if (is_exactly_a<numeric>(x))
-		return atan(ex_to<numeric>(x));
-	
-	return atan(x).hold();
-}
-
 static ex atan_eval(const ex & x)
 {
 	if (is_exactly_a<numeric>(x)) {
@@ -1306,7 +1232,7 @@ static ex atan_eval(const ex & x)
 			throw (pole_error("atan_eval(): logarithmic pole",0));
 
 		// atan(float) -> float
-		if (!x.info(info_flags::crational))
+		if (x.info(info_flags::inexact))
 			return atan(ex_to<numeric>(x));
 
 		// atan() is odd
@@ -1398,7 +1324,6 @@ static ex atan_conjugate(const ex & x)
 }
 
 REGISTER_FUNCTION(atan, eval_func(atan_eval).
-                        evalf_func(atan_evalf).
                         derivative_func(atan_deriv).
                         series_func(atan_series).
                         conjugate_func(atan_conjugate).
@@ -1467,8 +1392,8 @@ static ex atan2_eval(const ex & y, const ex & x)
 	}
 
 	// atan2(float, float) -> float
-	if (is_exactly_a<numeric>(y) && !y.info(info_flags::crational) &&
-	    is_exactly_a<numeric>(x) && !x.info(info_flags::crational))
+	if (is_exactly_a<numeric>(y) && y.info(info_flags::inexact) &&
+	    is_exactly_a<numeric>(x) && x.info(info_flags::inexact))
 		return atan(ex_to<numeric>(y), ex_to<numeric>(x));
 
 	// handle infinities
@@ -1525,6 +1450,7 @@ REGISTER_FUNCTION(atan2, eval_func(atan2_eval).
 // inverse cotangent (arc cotangent)
 //////////
 
+// Needed because there is no Python RR equivalent
 static ex acot_evalf(const ex & x, PyObject* parent)
 {
 	if (is_exactly_a<numeric>(x))
@@ -1549,7 +1475,7 @@ static ex acot_eval(const ex & x)
 		if (x.is_equal(I) || x.is_equal(-I))
 			throw (pole_error("acot_eval(): logarithmic pole",0));
 
-                if (!x.info(info_flags::crational))
+                if (x.info(info_flags::inexact))
                         return atan(ex_to<numeric>(x).inverse());
 
 		if (x.info(info_flags::negative))
@@ -1593,6 +1519,7 @@ REGISTER_FUNCTION(acot, eval_func(acot_eval).
 // inverse secant (arc secant)
 //////////
 
+// Needed because there is no Python RR equivalent
 static ex asec_evalf(const ex & x, PyObject* parent)
 {
 	if (is_exactly_a<numeric>(x))
@@ -1606,12 +1533,12 @@ static ex asec_eval(const ex & x)
 	if (is_exactly_a<numeric>(x)) {
                 numeric num = ex_to<numeric>(x);
                 if (num.is_zero())
-                        throw (pole_error("asec_eval(): asec(0) encountered",0));
+                        return NaN;
                 if (num.is_equal(*_num1_p))
                         return _ex0;
                 if (num.is_equal(*_num_1_p))
                         return Pi;
-                if (not num.info(info_flags::crational))
+                if (num.info(info_flags::inexact))
                         return acos(num.inverse());
 	}
 
@@ -1657,6 +1584,7 @@ REGISTER_FUNCTION(asec, eval_func(asec_eval).
 // inverse cosecant (arc cosecant)
 //////////
 
+// Needed because there is no Python RR equivalent
 static ex acsc_evalf(const ex & x, PyObject* parent)
 {
 	if (is_exactly_a<numeric>(x))
@@ -1670,12 +1598,12 @@ static ex acsc_eval(const ex & x)
 	if (is_exactly_a<numeric>(x)) {
                 numeric num = ex_to<numeric>(x);
                 if (num.is_zero())
-                        throw (pole_error("acsc_eval(): asec(0) encountered",0));
+                        return NaN;
                 if (num.is_equal(*_num1_p))
                         return Pi/_ex2;
                 if (num.is_equal(*_num_1_p))
                         return -Pi/_ex2;
-                if (not num.info(info_flags::crational))
+                if (num.info(info_flags::inexact))
                         return asin(num.inverse());
         }
 
