@@ -145,17 +145,17 @@ typedef std::map<ex2, exbool, ex2_less> ex2_exbool_remember;
 
 
 /** Return maximum (absolute value) coefficient of a polynomial.
- *  This function is used internally by heur_gcd().
+ *  This function was used internally by heur_gcd().
  *
  *  @return maximum coefficient
- *  @see heur_gcd */
+ */
 numeric ex::max_coefficient() const
 {
 	return bp->max_coefficient();
 }
 
 /** Implementation ex::max_coefficient().
- *  @see heur_gcd */
+ */
 numeric basic::max_coefficient() const
 {
 	return *_num1_p;
@@ -197,13 +197,42 @@ numeric mul::max_coefficient() const
 	return abs(ex_to<numeric>(overall_coeff));
 }
 
+bool ex::is_linear(const symbol& x, ex& a, ex& b) const
+{
+        if (degree(x) > 1)
+                return false;
+        b = coeff(x, 1);
+        if (has_symbol(b,x))
+                return false;
+        a = ((*this) - b*x).expand();
+        if (has_symbol(a,x))
+                return false;
+        return true;
+}
+
+bool ex::is_quadratic(const symbol& x, ex& a, ex& b, ex& c) const
+{
+        if (degree(x) > 2)
+                return false;
+        c = coeff(x, 2);
+        if (has_symbol(c,x))
+                return false;
+        b = coeff(x, 1);
+        if (has_symbol(b,x))
+                return false;
+        a = ((*this) - c*power(x,2) - b*x).expand();
+        if (has_symbol(a,x))
+                return false;
+        return true;
+}
+
 
 /** Apply symmetric modular homomorphism to an expanded multivariate
- *  polynomial.  This function is usually used internally by heur_gcd().
+ *  polynomial.  This function was usually used internally by heur_gcd().
  *
  *  @param xi  modulus
  *  @return mapped polynomial
- *  @see heur_gcd */
+ */
 ex basic::smod(const numeric &xi) const
 {
 	return *this;
@@ -389,7 +418,7 @@ static ex frac_cancel(const ex &n, const ex &d)
 	// Handle special cases where numerator or denominator is 0
 	if (num.is_zero())
 		return (new lst(num, _ex1))->setflag(status_flags::dynallocated);
-	if (den.expand().is_zero())
+	if (den.is_zero())
 		throw(std::overflow_error("frac_cancel: division by zero in frac_cancel"));
 
 	// Bring numerator and denominator to Z[X] by multiplying with
@@ -626,7 +655,7 @@ ex ex::normal(int level, bool noexpand_combined, bool noexpand_numer) const
 	if (!repl.empty())
 		e = e.subs(repl, subs_options::no_pattern);
 
-	// Convert {numerator, denominator} form back to fraction
+        // Convert {numerator, denominator} form back to fraction
         if ((options & normal_options::no_expand_fraction_numer) == 0u)
                 return e.op(0).expand() / e.op(1);
         else
@@ -1003,10 +1032,59 @@ ex gcd(const ex &a, const ex &b)
 {
         if (is_exactly_a<numeric>(a) && is_exactly_a<numeric>(b))
                 return gcd(ex_to<numeric>(a), ex_to<numeric>(b));
-        exmap repl;
-        ex poly_a = a.to_rational(repl);
-        ex poly_b = b.to_rational(repl);
-        return gcdpoly(poly_a, poly_b).subs(repl, subs_options::no_pattern);
+        return gcdpoly(a, b);
+}
+
+bool factor(const ex& the_ex, ex& res_ex)
+{
+        if (is_exactly_a<numeric>(the_ex)
+            or is_exactly_a<symbol>(the_ex)
+            or is_exactly_a<function>(the_ex)
+            or is_exactly_a<constant>(the_ex)) {
+                return false;
+        }
+        if (is_exactly_a<mul>(the_ex)) {
+                const mul& m = ex_to<mul>(the_ex);
+                exvector ev;
+                bool mchanged = false;
+                for (size_t i=0; i<m.nops(); ++i) {
+                        ex r;
+                        const ex& e = m.op(i);
+                        bool res = factor(e, r);
+                        if (res) {
+                                ev.push_back(r);
+                                mchanged = true;
+                        }
+                        else
+                                ev.push_back(e);
+                }
+                if (mchanged)
+                        res_ex = mul(ev);
+                return mchanged;
+        }
+        else if (is_exactly_a<power>(the_ex)) {
+                const power& p = ex_to<power>(the_ex);
+                ex r;
+                bool pchanged = factor(p.op(0), r);
+                if (pchanged)
+                        res_ex = power(r, p.op(1));
+                return pchanged;
+        }
+        ex num, den;
+        ex normalized = the_ex.numer_denom();
+        num = normalized.op(0);
+        bool nres = factorpoly(num, res_ex);
+        den = normalized.op(1);
+        ex res_den;
+        bool dres = factorpoly(den, res_den);
+        if (not nres and not dres)
+                return false;
+        if (not nres)
+                res_ex = num;
+        if (not dres)
+                res_den = den;
+        res_ex = res_ex / res_den;
+        return true;
 }
 
 } // namespace GiNaC
